/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _SRCINFO_H_
#define _SRCINFO_H_

#include "Object.h"
#include "ObjectParser.h"
#include "misc.h"
#include <cmath>

using namespace FD;

class SrcInfo : public Object{ 

 public:
  int id;          // Object ID
  float obj[3];    // 3D position of the object from the base sensor (x, y, z)
  float sensor[6]; // 3D position and orientation of the base sensor (x, y, z, roll, pitch, yaw)
  int img_size[2]; // Pixel size of the sensor image
  int roi_c[2];    // Center of the ROI in the image of base camera
  int roi_xy[2];   // Size of the ROI in the image of base camera
  double remaining;
   
  SrcInfo(){
  }

  SrcInfo(const SrcInfo& input) {
    id = input.id;
    for(int i = 0; i < 3; i++) obj[i]      = input.obj[i];
    for(int i = 0; i < 6; i++) sensor[i]   = input.sensor[i];
    for(int i = 0; i < 2; i++) img_size[i] = input.img_size[i];
    for(int i = 0; i < 2; i++) roi_c[i]    = input.roi_c[i];
    for(int i = 0; i < 2; i++) roi_xy[i]   = input.roi_xy[i];
    remaining = input.remaining;
  }
      
  ~SrcInfo(){
  }

  virtual ObjectRef clone()
  {
    return ObjectRef(new SrcInfo(*this));
  }
		
  /**Serialize (binary) the object to a stream*/
  virtual void serialize(std::ostream &out) const {
  }
  
  /**Unserialize (binary) the object from a stream*/
  virtual void unserialize(std::istream &in){
  }
		
  /**Generic print function*/
  virtual void printOn(std::ostream &out=std::cout) const {
    out << "<SrcInfo " << std::endl;
    out << "<SrcID " << id << " >" << std::endl;
    out << "<obj pos " << obj[0] << " " << obj[1] << " " << obj[2] << " >" << std::endl;
    out << "<roi c " << roi_c[0] << " " << roi_c[1] << " >" << std::endl;
    out << "<roi xy " << roi_xy[0] << " " << roi_xy[1] << " >" << std::endl;
    out << ">" << std::endl;
  }
     
  /**Generic read function*/
  virtual void readFrom(std::istream &in=std::cin){
  }      
      
};

#endif
